package com.bpcl.lpg.core.service.impl;

import org.apache.logging.log4j.LogManager;
import com.microsoft.azure.storage.blob.CloudBlockBlob;
import com.microsoft.azure.storage.blob.CloudBlobContainer;
import com.microsoft.azure.storage.blob.CloudBlobClient;
import java.io.IOException;
import com.microsoft.azure.storage.StorageException;
import java.security.InvalidKeyException;
import java.net.URISyntaxException;
import com.microsoft.azure.storage.CloudStorageAccount;
import java.util.UUID;
import java.time.format.DateTimeFormatter;
import java.time.LocalDate;
import java.time.ZoneId;
import org.springframework.web.multipart.MultipartFile;
import java.util.Date;
import java.util.Objects;
import java.util.Calendar;
import com.bpcl.lpg.core.model.LpgSafetyInspectionModel;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.text.MessageFormat;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.bpcl.lpg.facades.safetyInspection.data.SafetyInspectionRequestData;
import java.util.regex.Matcher;
import org.apache.commons.lang3.StringUtils;
import java.util.regex.Pattern;
import com.bpcl.lpg.core.dao.BpclSafetyInspectionDao;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.apache.logging.log4j.Logger;
import com.bpcl.lpg.core.service.BPCLSafetyInspectionService;

public class BPCLSafetyInspectionServiceImpl implements BPCLSafetyInspectionService
{
    private static final Logger LOG;
    private static final String DEBUG_JSON_FORMATTER = "Input json :: {0}";
    public static final String SAFETY_INSPECTION_CUSTOMER_DETAILS_API_NAME = "safety.inspection.customer.details.name";
    private static final String SAFETY_INSPECTION_START_PERIOD = "safety.inspection.search.start.date";
    private static final String MOBILE_REGX = "safety.inspection.mobile.regex";
    private static final String SAFETY_INSPEC_AZURE_STORAGE_CONNECTION = "safety.inspection.azure.storage.connection";
    private static final String SAFETY_INSPEC_AZURE_STORAGE_CONTAINER = "safety.inspection.azure.storage.container";
    private static final String ZONE_ID = "Asia/Kolkata";
    @Resource(name = "configurationService")
    private ConfigurationService configurationService;
    @Resource
    private KeyGenerator safetyInspectionKeyGenerator;
    @Resource
    private KeyGenerator safetyInspectionPartnerKeyGenerator;
    @Resource
    private LpgEncryptionService lpgEncryptionService;
    @Resource
    private LPGCredentialsService lpgCredentialsService;
    @Resource
    private BpclSafetyInspectionDao bpclSafetyInspectionDao;
    
    public boolean validMobileNumber(final String customerMobileNumber) {
        final Matcher matcher = Pattern.compile(this.configurationService.getConfiguration().getString("safety.inspection.mobile.regex")).matcher(customerMobileNumber);
        return matcher.matches() && StringUtils.isNumeric(customerMobileNumber) && customerMobileNumber.length() == 10;
    }
    
    public void populateRequestData(final SafetyInspectionRequestData requestData) {
        requestData.setUniqueAPIReqNo(this.safetyInspectionKeyGenerator.generate().toString());
        requestData.setPartnerTransId(this.safetyInspectionPartnerKeyGenerator.generate().toString());
        final String hashValue = this.getHashKeyValueRequest(requestData);
        requestData.setHashKey(hashValue);
        this.getDefaultValuesForNonMandatoryFields(requestData);
    }
    
    private void getDefaultValuesForNonMandatoryFields(final SafetyInspectionRequestData data) {
        if (StringUtils.isBlank(data.getEndPointName())) {
            data.setEndPointName("");
        }
        if (StringUtils.isBlank(data.getAddl1())) {
            data.setAddl1("");
        }
        if (StringUtils.isBlank(data.getAddl2())) {
            data.setAddl2("");
        }
        if (StringUtils.isBlank(data.getAddl3())) {
            data.setAddl3("");
        }
        if (StringUtils.isBlank(data.getConsumerNo())) {
            data.setConsumerNo("");
        }
        if (StringUtils.isBlank(data.getLpgId())) {
            data.setLpgId("");
        }
        if (StringUtils.isBlank(data.getDistCode())) {
            data.setDistCode("");
        }
        if (StringUtils.isBlank(data.getContactNumber())) {
            data.setContactNumber("");
        }
        if (StringUtils.isBlank(data.getContactNumberType())) {
            data.setContactNumberType("");
        }
    }
    
    private String getHashKeyValueRequest(final SafetyInspectionRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = this.configurationService.getConfiguration().getString("safety.inspection.customer.details.name");
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartnerCode(partnerCode);
        hashMessage.append(this.getEmptyIfBlank(partnerCode));
        hashMessage.append(this.getEmptyIfBlank(requestData.getPartnerTransId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getUniqueAPIReqNo()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumberType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumber()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getLpgId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerNo()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl1()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl2()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl3()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        final String encryptedMsg = this.lpgEncryptionService.encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private String getEmptyIfBlank(final String stringToCheck) {
        return StringUtils.isBlank(stringToCheck) ? "" : stringToCheck;
    }
    
    public String getRequestJsonForsafetyInspection(final SafetyInspectionRequestData requestData) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(requestData);
        if (BPCLSafetyInspectionServiceImpl.LOG.isDebugEnabled()) {
            BPCLSafetyInspectionServiceImpl.LOG.debug(MessageFormat.format("Input json :: {0}", json));
        }
        return objectMapper.writeValueAsString(requestData);
    }
    
    public LpgSafetyInspectionModel isInspectionDetailsExist(final String consumerNo, final String distCode, final String contactNum, final String lpgId) {
        LpgSafetyInspectionModel lpgSafetyInspectionModel = null;
        final Calendar cal = Calendar.getInstance();
        cal.add(10, -this.configurationService.getConfiguration().getInt("safety.inspection.search.start.date"));
        final Date startDate = cal.getTime();
        if (StringUtils.isNotBlank(consumerNo)) {
            lpgSafetyInspectionModel = this.bpclSafetyInspectionDao.getInspectionDetailsByConsumerNo(consumerNo, distCode, startDate);
        }
        if (StringUtils.isNotBlank(contactNum)) {
            lpgSafetyInspectionModel = this.bpclSafetyInspectionDao.getInspectionDetailsByContactNumber(contactNum, startDate);
        }
        if (StringUtils.isNotBlank(lpgId)) {
            lpgSafetyInspectionModel = this.bpclSafetyInspectionDao.getInspectionDetailsByLpgID(lpgId, startDate);
        }
        if (Objects.nonNull(lpgSafetyInspectionModel)) {
            return lpgSafetyInspectionModel;
        }
        return null;
    }
    
    public String uploadSafetyInspectionImage(final MultipartFile file, final String lpgId) {
        final String connectionString = this.configurationService.getConfiguration().getString("safety.inspection.azure.storage.connection");
        final String containerName = this.configurationService.getConfiguration().getString("safety.inspection.azure.storage.container");
        final LocalDate dt = LocalDate.now(ZoneId.of("Asia/Kolkata"));
        final DateTimeFormatter yearFormatter = DateTimeFormatter.ofPattern("yyyy");
        final DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MMM");
        final DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        final String year = dt.format(yearFormatter);
        final String month = dt.format(monthFormatter);
        final String day = dt.format(dayFormatter);
        final String blobFolder = String.format("%s/%s/%s/", year, month, day);
        final String fileExt = org.springframework.util.StringUtils.getFilenameExtension(file.getOriginalFilename());
        final String uuid = UUID.randomUUID().toString().split("-")[0];
        final String fileName = lpgId + "-" + uuid + "." + fileExt;
        try {
            final CloudStorageAccount storageAccount = CloudStorageAccount.parse(connectionString);
            final CloudBlobClient client = storageAccount.createCloudBlobClient();
            final CloudBlobContainer container = client.getContainerReference(containerName);
            final CloudBlockBlob blob = container.getBlockBlobReference(blobFolder + fileName);
            blob.upload(file.getInputStream(), file.getSize());
            return blob.getStorageUri().getPrimaryUri().toString();
        }
        catch (final URISyntaxException | InvalidKeyException | StorageException | IOException e) {
            BPCLSafetyInspectionServiceImpl.LOG.error("Error occured while uploading the safety inspection image to Blob storage ", (Object)e.getMessage());
            return "";
        }
    }
    
    static {
        LOG = LogManager.getLogger((Class)BPCLSafetyInspectionServiceImpl.class);
    }
}
